package com.example.demo;

import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.games.model.GameEntity;
import com.example.demo.games.service.GameService;

import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	private final TypeService typeService;

	private final GameService gameService;

	private final UserService userService;

	// private final ItemService itemService;

	public DemoApplication(TypeService typeService, GameService gameService, UserService userService) {
		this.typeService = typeService;
		this.gameService = gameService;
		this.userService = userService;
	}

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	// Создавать данные
	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types");
			final var type1 = typeService.create(new TypeEntity(null, "FMV"));
			final var type2 = typeService.create(new TypeEntity(null, "Экшен"));
			final var type3 = typeService.create(new TypeEntity(null, "Стратегия"));

			log.info("Create default games");
			gameService.create(new GameEntity(null, type1, "Пробная игра 1", 100, "Пробное описание 1"));
			gameService.create(new GameEntity(null, type2, "Пробная игра 2", 200, "Пробное описание 2"));
			gameService.create(new GameEntity(null, type3, "Пробная игра 3", 300, "Пробное описание 3"));

			log.info("Create default users");
			userService.create(new UserEntity(null, "Пробный пользователь 1", "proba1@mail.ru", "password1"));
			userService.create(new UserEntity(null, "Пробный пользователь 2", "proba2@mail.ru", "password2"));
		}
	}
}
