package com.example.demo.games.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.games.model.GameEntity;
import com.example.demo.games.service.GameService;
import com.example.demo.types.service.TypeService;

import jakarta.validation.Valid;


@RestController
@RequestMapping(Constants.API_URL + "/game")
public class GameController {
    private final GameService gameService;
    private final TypeService typeService;
    private final ModelMapper modelMapper;

    public GameController(GameService gameService, TypeService typeService, ModelMapper modelMapper) {
        this.gameService = gameService;
        this.typeService = typeService;
        this.modelMapper = modelMapper;
    }

    private GameDto toDto(GameEntity entity) {
        return modelMapper.map(entity, GameDto.class);
    }

    private GameEntity toEntity(GameDto dto) {
        final GameEntity entity = modelMapper.map(dto, GameEntity.class);
        entity.setType(typeService.get(dto.getTypeId()));
        return entity;
    }

    //Получение всех игр по id жанра, по умолчанию id = 0 => (из бизнес-логики) вывод игр всех жанров по умолчанию
    @GetMapping
    public List<GameDto> getAll(@RequestParam(name = "typeId", defaultValue = "0") Long typeId) {
        return gameService.getAll(typeId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public GameDto get(@PathVariable(name = "id") Long id) {
        return toDto(gameService.get(id));
    }


    @PostMapping
    public GameDto create(@RequestBody @Valid GameDto dto) {
        return toDto(gameService.create(toEntity(dto)));
    }


    @PutMapping("/{id}")
    public GameDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid GameDto dto) {
        return toDto(gameService.update(id, toEntity(dto)));
    }

    
    @DeleteMapping("/{id}")
    public GameDto delete(@PathVariable(name = "id") Long id) {
        return toDto(gameService.delete(id));
    }

}
