package com.example.demo.games.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.games.model.GameEntity;
import com.example.demo.games.repository.GameRepository;


@Service
public class GameService {
    private final GameRepository repository;

    public GameService(GameRepository repository) {
        this.repository = repository;
    }
    
    //ПОЛУЧЕНИЕ ВСЕХ ИГР (С ВОЗМОЖНОЙ ФИЛЬТРАЦИЕЙ ПО ЖАНРУ (по его id))
    //МБ СДЕЛАТЬ отдельный getAll для получения всех игр всех жанров
    public List<GameEntity> getAll(Long typeId) {
        //Если передан typeId=0, то возвращение всех игр (всех жанров) 
        if (Objects.equals(typeId, 0L)) {
            return repository.getAll();
        }
        // возвращение всех игр, у которых у жанра тот же id, что и был передан
        return repository.getAll().stream().filter(item -> item.getType().getId().equals(typeId)).toList();
    }

    public GameEntity get(Long id) {
        // Если объект не null, вернёт его; если null, вернёт кастомное исключение
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public GameEntity create(GameEntity entity) {
        return repository.create(entity);
    }

    public GameEntity update(Long id, GameEntity entity) {
        final GameEntity existedEntity = get(id);

        existedEntity.setType(entity.getType());
        existedEntity.setTitle(entity.getTitle());
        existedEntity.setPrice(entity.getPrice());
        existedEntity.setDescription(entity.getDescription());

        return repository.update(existedEntity);
    }

    public GameEntity delete(Long id) {
        final GameEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
