package com.example.demo.types.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.repository.TypeRepository;

//Бизнес-логика жанров (типов) игр

@Service
public class TypeService {
    // Словарь со всеми жанрами
    private final TypeRepository repository;

    public TypeService(TypeRepository repository) {
        this.repository = repository;
    }

    public List<TypeEntity> getAll() {
        return repository.getAll();
    }

    public TypeEntity get(Long id) {
        // Если объект не null, вернёт его; если null, вернёт кастомное исключение
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public TypeEntity create(TypeEntity entity) {
        return repository.create(entity);
    }

    public TypeEntity update(Long id, TypeEntity entity) {
        //Получение жанра по id
        final TypeEntity existedEntity = get(id);
        //Установка такого же имени, как у переданного жанра
        existedEntity.setName(entity.getName());
        //Обновление жанра с переданным id в словаре
        return repository.update(existedEntity);
    }

    public TypeEntity delete(Long id) {
        //Получение жанра по id
        final TypeEntity existedEntity = get(id);
        //Удаление этого жанра из словаря
        return repository.delete(existedEntity);
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
