package com.example.demo.users.model;

import java.util.Objects;
import java.util.Set;
import java.util.LinkedHashSet;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.games.model.GameEntity;

// Сама сущность пользователя
public class UserEntity extends BaseEntity {
    private String name;
    private String email;
    private String password;
    private final Set<GameEntity> games = new LinkedHashSet<>();

    public UserEntity() {
        super();
    }

    public UserEntity(Long id, String name, String email, String password) {
        super(id);
        this.name = name;
        this.email = email;
        this.password = password;
    }

    //Получение - установление значения имени (ника) пользователя
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    //Получение - установление значения почты пользователя
    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }


    //Получение - установление значения пароля пользователя
    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }


    public void addGame(GameEntity game) {
        games.add(game);
    }

    public void deleteGame(GameEntity game) {
        games.remove(game);
    }

    public Set<GameEntity> getGames() {
        return games;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name);
    }

    //Сравнение 2 пользователей
    //перегрузка метода equals 
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
            
        // Если пришедший объект пустой или другого класса, то пользователи не равны
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
      
        final UserEntity second = (UserEntity) obj;
        //Сравнение по id, нику, почте, паролю
        return Objects.equals(second.getId(), id) && Objects.equals(second.getName(), name) 
        && Objects.equals(second.getEmail(), email) && Objects.equals(second.getPassword(), password)
        && second.getGames().equals(games);
    }

}
