package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.games.model.GameEntity;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

//Бизнес-логика пользователей

@Service
public class UserService {
    // Словарь со всеми пользователями
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    public List<UserEntity> getAll() {
        return repository.getAll();
    }

    public UserEntity get(Long id) {
        // Если объект не null, вернёт его; если null, вернёт кастомное исключение
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public UserEntity create(UserEntity entity) {
        return repository.create(entity);
    }

    public UserEntity update(Long id, UserEntity entity) {
        //Получение пользователя по id
        final UserEntity existedEntity = get(id);

        //Установка такого же имени, как у переданного пользователя
        existedEntity.setName(entity.getName());

        //Установка такой же почты, как у переданного пользователя
        existedEntity.setEmail(entity.getEmail());

        //Установка такого же пароля, как у переданного пользователя
        existedEntity.setPassword(entity.getPassword());

        //Обновление пользователя с переданным id в словаре
        return repository.update(existedEntity);
    }

    public UserEntity delete(Long id) {
        //Получение жанра по id
        final UserEntity existedEntity = get(id);
        //Удаление этого жанра из словаря
        return repository.delete(existedEntity);
    }

    public UserEntity addGame(Long id, GameEntity game) {
        final UserEntity existedEntity = get(id);
        existedEntity.addGame(game);
        return existedEntity;
    }

    public UserEntity deleteGame(Long id, GameEntity game) {
        final UserEntity existedEntity = get(id);
        existedEntity.deleteGame(game);
        return existedEntity;
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
