package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class TypeServiceTests {
	//Автоматическое внедрение зависимостей в сервис
	@Autowired
	private TypeService typeService;

	//Проверка того, что get выдаст кастомное исключение, если в него передать несуществующий id 
	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> typeService.get(0L));
	}

	//Проверка добавления в хранилище
	@Test
	@Order(1)
	void createTest() {
		typeService.deleteAll();
		typeService.create(new TypeEntity(null, "FMV"));
		typeService.create(new TypeEntity(null, "Экшен"));
		final TypeEntity last = typeService.create(new TypeEntity(null, "Стратегия"));

		//Проверки, что значения равны. Первый параметр - эталон (ожидаемое значение), второй - получаемое значение
		//проверка, что кол-во жанров в хранилище равно кол-ву добавленных (проверка, что все добавились)
		
		Assertions.assertEquals(3, typeService.getAll().size());
		//проверка, что в хранилище последний жанр тот жанр, который добавили последним (проверка, что всё правильно добавилось)
		Assertions.assertEquals(last, typeService.get(3L));
	}

	//Проверка правильности обновления на примере последнего жанра
	@Test
	@Order(2)
	void updateTest() {
		//на какое название будем менять название жанра
		final String test = "TEST";
		//получение последнего в списке жанра
		final TypeEntity entity = typeService.get(3L);
		//запоминание его названия до обновления
		final String oldName = entity.getName();
		//запоминание новых данных жанра
		final TypeEntity newEntity = typeService.update(3L, new TypeEntity(1L, test));
		//Проверка, что название поменялось
		Assertions.assertEquals(test, newEntity.getName());
		Assertions.assertNotEquals(oldName, newEntity.getName());
	}

	//Проверка правильности удаления на примере последнего жанра
	@Test
	@Order(3)
	void deleteTest() {
		typeService.delete(3L);
		//Проверка, что после удаления в хранилище осталось 2 жанра
		Assertions.assertEquals(2, typeService.getAll().size());
	
		final TypeEntity newEntity = typeService.create(new TypeEntity(null, "Стратегия"));
		//Проверка, что в хранилище теперь снова 3 жанра
		Assertions.assertEquals(3, typeService.getAll().size());
		//Проверка, что id у добавленного жанра стал 4
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
