package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.DependsOn;

import java.util.List;
import java.util.ArrayList;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.games.model.GameEntity;
import com.example.demo.games.service.GameService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
@DependsOn("GameServiceTests")
public class UserServiceTests {

    @Autowired
	private UserService userService;

    @Autowired
	private GameService gameService;

	@Autowired
	private TypeService typeService;

    //Проверка того, что get выдаст кастомное исключение, если в него передать несуществующий id 
	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
	}

    //Проверка добавления в хранилище
    @Test
	@Order(1)
	void createTest() {
        userService.deleteAll();
		gameService.deleteAll();
		typeService.deleteAll();

		typeService.create(new TypeEntity(null, "FMV"));
		typeService.create(new TypeEntity(null, "Экшен"));
		typeService.create(new TypeEntity(null, "Стратегия"));

		gameService.create(new GameEntity(null, typeService.get(1L), "Пробная игра 1", 100, "Пробное описание 1"));
		gameService.create(new GameEntity(null, typeService.get(2L), "Пробная игра 2", 450, "Пробное описание 2"));
        gameService.create(new GameEntity(null, typeService.get(3L), "Пробная игра 3", 330, "Пробное описание 3"));

		userService.create(new UserEntity(null, "Пробный ник 1", "Пробная_почта_1@mail.ru", "Пробный пароль 1"));
        userService.create(new UserEntity(null, "Пробный ник 2", "Пробная_почта_2@mail.ru", "Пробный пароль 2"));

        Assertions.assertEquals(2, userService.getAll().size());
	}

    //Проверка правильности обновления на примере первого
    @Test
	@Order(2)
	void updateTest() {
        //на какой ник будем менять ник пользователя
		final String testName = "TEST";

        //на какую почту будем менять почту пользователя
        final String testEmail = "test@mail.ru";

        //на какой пароль будем менять пароль пользователя
        final String testPassword = "TEST PASSWORD";

		//запоминание обновлённого пользователя
        final UserEntity newEntity = userService.update(1L, new UserEntity(1L, testName, testEmail, testPassword));

		//Проверка, что кол-во пользователей до сих пор 2
		Assertions.assertEquals(2, userService.getAll().size());

		Assertions.assertEquals(newEntity, userService.get(1L));

		//Проверка, что у пользователя нет игр
		Assertions.assertEquals(0, userService.get(1L).getGames().size());

		//Добавление пользователю 2 игр
        userService.addGame(1L, gameService.get(3L));
		userService.addGame(1L, gameService.get(2L));
		//Проверка, что у пользователя теперь есть 2 игры
        Assertions.assertEquals(2, userService.get(1L).getGames().size());

		//Удаление у пользователя игры
        userService.deleteGame(1L, gameService.get(2L));
		//Проверка, что у пользователя 1 игра
        Assertions.assertEquals(1, userService.get(1L).getGames().size());
	}
    
    //Проверка правильности удаления
	@Test
	@Order(3)
	void deleteTest() {
		userService.delete(1L);
		//Проверка, что после удаления в хранилище осталcя пользователей
		Assertions.assertEquals(1, userService.getAll().size());

		userService.create(new UserEntity(null, "Пробный ник 1", "Пробная_почта_1@mail.ru", "Пробный пароль 1"));
		//Проверка, что в хранилище теперь снова 2 пользователя
		Assertions.assertEquals(2, userService.getAll().size());
	}
}
